import { GraphQLError } from 'graphql';
import { IResolvers, withFilter } from 'mercurius';

export const MESSAGE_ADDED = 'MESSAGE_ADDED';

export const messageResolvers: IResolvers = {
  Query: {
    messages: async (_, { roomId }, { prisma, userId }) => {
      if (!userId) {
        throw new GraphQLError('You must be logged in to view messages', {
          extensions: {
            code: 'UNAUTHENTICATED',
          },
        });
      }

      // Check if user is a member of the room
      const room = await prisma.room.findUnique({
        where: { id: roomId },
        include: { members: true },
      });

      if (!room) {
        throw new GraphQLError('Room not found', {
          extensions: {
            code: 'NOT_FOUND',
          },
        });
      }

      const isMember = room.members.some(
        (member: { id: string }) => member.id === userId
      );
      if (!isMember) {
        throw new GraphQLError('You are not a member of this room', {
          extensions: {
            code: 'FORBIDDEN',
          },
        });
      }

      return prisma.message.findMany({
        where: { roomId },
        orderBy: { createdAt: 'asc' },
      });
    },
  },
  Mutation: {
    sendMessage: async (
      _: any,
      { content, roomId }: { content: string; roomId: string },
      { prisma, userId, pubsub }
    ) => {
      if (!userId) {
        throw new GraphQLError('You must be logged in to send a message', {
          extensions: {
            code: 'UNAUTHENTICATED',
          },
        });
      }

      // Check if user is a member of the room
      const room = await prisma.room.findUnique({
        where: { id: roomId },
        include: { members: true },
      });

      if (!room) {
        throw new GraphQLError('Room not found', {
          extensions: {
            code: 'NOT_FOUND',
          },
        });
      }

      const isMember = room.members.some(
        (member: { id: string }) => member.id === userId
      );
      if (!isMember) {
        throw new GraphQLError('You are not a member of this room', {
          extensions: {
            code: 'FORBIDDEN',
          },
        });
      }

      const message = await prisma.message.create({
        data: {
          content,
          user: {
            connect: { id: userId },
          },
          room: {
            connect: { id: roomId },
          },
        },
        include: {
          user: true,
          room: true,
        },
      });

      pubsub.publish({
        topic: MESSAGE_ADDED,
        payload: { messageAdded: message, roomId },
      });

      return message;
    },
  },
  Subscription: {
    messageAdded: {
      subscribe: withFilter(
        (_, __, { pubsub }) => pubsub.subscribe([MESSAGE_ADDED]),
        (payload, variables) => {
          return payload.roomId === variables.roomId;
        }
      ),
    },
  },
  Message: {
    user: async (parent, _, { prisma }) => {
      if (parent.user) {
        return parent.user;
      }
      return prisma.user.findUnique({
        where: { id: parent.userId },
      });
    },
    room: async (parent, _, { prisma }) => {
      if (parent.room) {
        return parent.room;
      }
      return prisma.room.findUnique({
        where: { id: parent.roomId },
      });
    },
  },
};
